import matplotlib as mpl
import numpy as np
import inspiral_range as ir

from matplotlib import pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from scipy.interpolate import interp1d
from scipy.integrate import cumtrapz

plt.rcdefaults()
mpl.rcParams.update({
    'savefig.pad_inches': 0.02,
    'text.usetex': True,
    'text.latex.preamble': r'\usepackage{amsmath,lxfonts,upgreek}',
    })

def makegrid(ax):
    ax.grid(True, which='major', color='k', alpha=0.05)
    ax.grid(True, which='minor', color='k', alpha=0.02)

text_bbox = dict(fc='w', alpha=0.7, lw=0, pad=0.1)

# %%
fO3_H1, aO3_H1 = np.loadtxt(
    'O3-H1-C01_CLEAN_SUB60HZ-1262197260.0_sensitivity_strain_asd_binned.txt',
    unpack=1,
    )
fO3_L1, aO3_L1 = np.loadtxt(
    'O3-L1-C01_CLEAN_SUB60HZ-1262141640.0_sensitivity_strain_asd_binned.txt',
    unpack=1,
    )
fO4_H1, aO4_H1 = np.loadtxt(
    'O4a-H1_2024-01-11_070000_Z_binned.txt',
    unpack=1,
    )
fO4b_H1, aO4b_H1 = np.loadtxt(
    'PreO4b-H1_2024-03-16_104729_Z_binned.txt',
    unpack=1,
    )
fO4_L1, _, aO4_L1 = np.loadtxt(
    'O4-L1_2023-06-22_044444_Z_binned.txt',
    unpack=1,
    )
fgwinc, agwinc = np.loadtxt(
    'gwinc_O4_400kW_4p5dB_FDS.txt',
    unpack=1,
    )

O3_H1_kwargs = dict(
    label='H1 O3 (LIGO--G2100674)',
    color='xkcd:burgundy',
    alpha=0.9,
    zorder=25,
    )

O3_L1_kwargs = dict(
    label='L1 O3 (LIGO--G2100675)',
    color='xkcd:navy blue',
    alpha=0.9,
    zorder=20,
    )

O4_H1_kwargs = dict(
    label='H1 O4a (2024--01--11 07:00:00 Z)',
    color='xkcd:mulberry',
    alpha=0.9,
    zorder=35,
    )
O4b_H1_kwargs = dict(
    label='H1 pre-O4b (2024--03--16 10:47:29 Z)',
    color='xkcd:strawberry',
    alpha=0.9,
    zorder=35,
    )

O4_L1_kwargs = dict(
    label='L1 O4a (2023--06--22 04:44:44 Z)',
    color='xkcd:cerulean',
    alpha=0.9,
    zorder=30,
    )

gwinc_kwargs = dict(
    label='GWINC (400 kW, 4.5 dB FDS)',
    color='xkcd:blue grey',
    alpha=0.9,
    zorder=5,
    )

O3_H1_dict = dict(
    freq=fO3_H1,
    asd=aO3_H1,
    lw=1,
    kwargs=O3_H1_kwargs,
    )

O3_L1_dict = dict(
    freq=fO3_L1,
    asd=aO3_L1,
    lw=1,
    kwargs=O3_L1_kwargs,
    )

O4_H1_dict = dict(
    freq=fO4_H1,
    asd=aO4_H1,
    lw=1,
    kwargs=O4_H1_kwargs,
    )

O4b_H1_dict = dict(
    freq=fO4b_H1,
    asd=aO4b_H1,
    lw=1,
    kwargs=O4b_H1_kwargs,
    )

O4_L1_dict = dict(
    freq=fO4_L1,
    asd=aO4_L1,
    lw=1,
    kwargs=O4_L1_kwargs,
    )

gwinc400kW_dict = dict(
    freq=fgwinc,
    asd=agwinc,
    lw=2,
    kwargs=gwinc_kwargs
    )

traces_list = [
    #O3_L1_dict,
    #O3_H1_dict,
    O4_L1_dict,
    O4_H1_dict,
    O4b_H1_dict,
    gwinc400kW_dict,
    ]

# %%
fig_strain, ax_strain = plt.subplots(figsize=(4, 3))
fig_Wgw, ax_Wgw = plt.subplots(figsize=(4, 3))

for td in traces_list:
    ax_strain.loglog(
        td['freq'],
        td['asd'],
        lw=td['lw'],
        **td['kwargs'],
        )
    ax_Wgw.loglog(
        td['freq'],
        (2 * np.pi**2 / 3) * td['freq']**3 * td['asd']**2,
        lw=td['lw'],
        **td['kwargs'],
        )

ax_strain.set_ylim([2e-24, 1.5e-21])
ax_strain.set_ylabel(r'Strain noise $\bigl[1\big/\!\sqrt{\text{Hz}}\bigr]$')

ax_Wgw.set_ylim([7e-42, 1.5e-36])
ax_Wgw.set_ylabel(r'$(2\pi^2\!/3)\,f^3\,S_{hh}(f)$ [Hz$^2$]')

for ax in [ax_strain, ax_Wgw]:
    ax.set_xlim([10, 5e3])
    ax.xaxis.set_major_formatter(mpl.ticker.FuncFormatter(lambda y, _: '{:g}'.format(y)))
    ax.set_xlabel('Frequency [Hz]')
    ax.legend(
        fontsize='small',
        handlelength=1.5,
        loc='upper right',
        ).set_zorder(100)
    makegrid(ax)

fig_strain.savefig('O4a_preO4b_strain.pdf', bbox_inches='tight')
fig_Wgw.savefig('O4a_preO4b_Wgw.pdf', bbox_inches='tight')

# %%
# Horizon estimates
marr = np.geomspace(1, 2500, 120)

fig_zhor, ax_zhor = plt.subplots(figsize=(4, 3))
fig_volhor, ax_volhor = plt.subplots(figsize=(4, 3))
fig_volhorrat, ax_volhorrat = plt.subplots(figsize=(4, 3))

for td in traces_list:
    td['zhor'] = np.zeros_like(marr)
    td['volhor'] = np.zeros_like(marr)
    for i, m in enumerate(marr):
        lal_kwargs = {
        'm1': m/2,
        'm2': m/2,
        #'inclination': np.pi/4,
        'approximant': 'IMRPhenomD',
        }
        td['zhor'][i] = ir.horizon_redshift(td['freq'], td['asd']**2, **lal_kwargs)
        td['volhor'][i] = ir.volume(td['freq'], td['asd']**2, **lal_kwargs) / 1e9
    ax_zhor.loglog(
        marr,
        td['zhor'],
        lw=2,
        **td['kwargs'],
        )
    ax_volhor.loglog(
        marr,
        td['volhor'],
        lw=2,
        **td['kwargs'],
        )

for ax in [ax_volhor, ax_volhorrat]:
#    lL1_O4, = ax.loglog(
#        marr[marr<1500],
#        (O4_L1_dict['volhor'] / O3_L1_dict['volhor'])[marr<1500],
#        lw=1.5,
#        ls=(3, (4.5, 1.5,)),
#        color=O4_L1_kwargs['color'],
#        )
    lH1_O4, = ax.loglog(
        marr[marr<1500],
        (O4b_H1_dict['volhor'] / O4_H1_dict['volhor'])[marr<1500],
        lw=1.5,
        ls=(3, (4.5, 1.5,)),
        color=O4b_H1_kwargs['color'],
        )

for ax in [ax_zhor, ax_volhor, ax_volhorrat]:
    ax.set_xlim([marr[0], marr[-1]])
    ax.set_xlabel(r'Total source-frame mass [$M_\odot$]')
    ax.xaxis.set_major_formatter(mpl.ticker.FuncFormatter(lambda y, _: '{:g}'.format(y)))
    ax.yaxis.set_major_formatter(mpl.ticker.FuncFormatter(lambda y, _: '{:g}'.format(y)))
    ax.text(
        0.05,
        0.95,
        r'SNR threshold = 8',
        ha='left',
        va='top',
        fontsize='small',
        transform=ax.transAxes,
        bbox=text_bbox,
        )
    ax.legend(
        fontsize='xx-small',
        handlelength=1.25,
        labelspacing=0.15,
        ).set_zorder(2)
    makegrid(ax)

h, l = ax_volhor.get_legend_handles_labels()
h.append(
    mpl.lines.Line2D([], [], color='xkcd:black', ls=(3, (4.5, 1.5),), lw=2))
l.append('Volume ratio pre-O4b / O4a')
leg_volhor = ax_volhor.legend(
    h,
    l,
    fontsize='x-small',
    loc='lower center',
    bbox_to_anchor=(0.57, 0),
    handlelength=1.25,
    labelspacing=0.15,
    )
h, l = ax_volhorrat.get_legend_handles_labels()
leg_volhorrat = ax_volhorrat.legend(
    [
    #    lL1_O4,
        lH1_O4,
    ],
    [
    #    'L1',
        'H1',
    ],
    loc='lower left',
    )

ax_zhor.set_ylim([0.02, 3])
ax_zhor.set_ylabel('Horizon redshift')

ax_volhor.set_ylim([3e-3, 150])
ax_volhor.set_ylabel('Comoving horizon volume [Gpc$^3$]')

ax_volhorrat.set_yscale('linear')
ax_volhorrat.yaxis.set_minor_locator(mpl.ticker.MultipleLocator(0.2))
ax_volhorrat.set_ylim([0, 5])
ax_volhorrat.set_ylabel('Ratio of comoving volume')

fig_zhor.savefig('O4a_preO4b_horizon_redshift.pdf', bbox_inches='tight')
fig_volhor.savefig('O4a_preO4b_comoving_volume.pdf', bbox_inches='tight')
fig_volhorrat.savefig('O4a_preO4b_comoving_volume_ratio.pdf', bbox_inches='tight')

def logtrp(x, y):
    with np.errstate(divide='ignore'):
        logx = np.log10(x)
        logy = np.log10(y)
    logtrpfunc = interp1d(logx, logy)
    trpfunc = lambda a: 10**(logtrpfunc(np.log10(a)))
    return trpfunc

dz0p1 = 463e6 # pc; z = 0.1
mlist = [1.4, 3, 10, 30, 100, 300, 1000,] # Msol
with PdfPages('O4a_preO4b_range_integrands.pdf') as range_pdf:
    for i, m in enumerate(mlist):
        fwf, hpwf, _ = ir.waveform.gen_waveform(m1=m, m2=m, distance=dz0p1)
        wf_terp = logtrp(fwf, np.abs(hpwf))
        fig_ri, ax_ri = plt.subplots(figsize=(4, 3))
        for td in traces_list:
            ax_ri.semilogx(
                td['freq'],
                np.sqrt(4 * td['freq'] * wf_terp(td['freq'])**2 / td['asd']**2),
                **td['kwargs'],
            )
        ax_ri.text(
            0.98,
            0.625,
            '$m_1 = m_2 = {} M_\odot$'.format(m),
            ha='right',
            va='center',
            fontsize='small',
            transform=ax_ri.transAxes,
            bbox=text_bbox,
            )
        ax_ri.text(
            0.98,
            0.575,
            '$z = 0.1$',
            ha='right',
            va='center',
            fontsize='small',
            transform=ax_ri.transAxes,
            bbox=text_bbox,
            )
        ax_ri.set_xlim([10, 5e3])
        #ymin, ymax = ax_ri.get_ylim()
        #ax_ri.set_ylim([0.01, ymax])
        ax_ri.xaxis.set_major_formatter(mpl.ticker.FuncFormatter(lambda y, _: '{:g}'.format(y)))
        ax_ri.yaxis.set_major_formatter(mpl.ticker.FuncFormatter(lambda y, _: '{:g}'.format(y)))
        ax_ri.set_xlabel('Frequency [Hz]')
        ax_ri.set_ylabel(r'$2\sqrt{f}\bigl\lvert h(f) \bigr\rvert \big/\!\sqrt{S_{hh}(f)}$ '
            r'$\bigl[1\big/\!\sqrt{\text{Hz}}\bigr]$')
        ax_ri.legend(
            fontsize='x-small',
            handlelength=1.0,
            loc='upper right',
            ).set_zorder(1)
        makegrid(ax_ri)
        range_pdf.savefig(fig_ri, bbox_inches='tight')
        plt.close(fig_ri)
